# -*- coding: utf-8 -*-

from __future__ import absolute_import
from django.conf.urls import patterns, include, url

from .views import ProvincesForCountry, CitiesForCountry

urlpatterns = patterns('',
    url(r'regions/$', ProvincesForCountry.as_view(), name="regions"),
    url(r'cities/$', CitiesForCountry.as_view(), name="cities"),
)



# -*- coding: utf-8 -*-

import io
import os
import sys
import os.path
import optparse
import zipfile
import requests

from django.core.management.base import BaseCommand
from django.utils.encoding import force_unicode
from django.db import transaction
from django.conf import settings

from ...models import City, Country, Region
from ... import settings as local_settings

PO_HEADER = u"""
msgid ""
msgstr ""
"Project-Id-Version: PACKAGE VERSION\\n"
"Report-Msgid-Bugs-To: \\n"
"POT-Creation-Date: 2012-06-13 16:29+0200\\n"
"PO-Revision-Date: YEAR-MO-DA HO:MI+ZONE\\n"
"Last-Translator: FULL NAME <EMAIL@ADDRESS>\\n"
"Language-Team: LANGUAGE <LL@li.org>\\n"
"Language: \\n"
"MIME-Version: 1.0\\n"
"Content-Type: text/plain; charset=UTF-8\\n"
"Content-Transfer-Encoding: 8bit\\n"
"Plural-Forms: nplurals=2; plural=(n != 1)\\n"
"""


class Command(BaseCommand):
    option_list = BaseCommand.option_list + (
        optparse.make_option('--force', action='append', default=[],
            help='Download and import even if matching files are up-to-date'
        ),
    )

    def prepare(self):
        if not os.path.exists(local_settings.TMPDATA_DIR):
            print "Creating temporal data directory: {0}".format(local_settings.TMPDATA_DIR)
            os.mkdir(local_settings.TMPDATA_DIR)

        # Generate all ids
        self.data = {}
        for pk, slug in Country.objects.values_list('geoname_id', 'name'):
            self.data[pk] = slug

        for item in Region.objects.values_list('geoname_id', 'name'):
            self.data[pk] = slug

        for item in City.objects.values_list('geoname_id', 'name'):
            self.data[pk] = slug

        self.langs = dict(settings.LANGUAGES).keys()

    def download(self, name, source, dest_path):
        response = requests.get(source)
        file_size = int(response.headers['content-length'])

        with io.open(dest_path, "bw") as f:
            for chuck in response.iter_content(512):
                f.write(chuck)
                sys.stdout.write(".")
            sys.stdout.write("\n")

    def extract(self, infile, outfile, filename):
        zfile = zipfile.ZipFile(infile)
        if zfile:
             with io.open(outfile, "wb") as f:
                f.write(zfile.read(filename))

    def download_datafiles(self):
        source = local_settings.TRANSLATIONS['source']
        filename = local_settings.TRANSLATIONS['file']
        dest_name = local_settings.TRANSLATIONS['dest']

        print "Downloading: {0}".format(dest_name)

        dest_path = os.path.join(local_settings.TMPDATA_DIR, dest_name)
        if os.path.exists(dest_path):
            print "File exists, skiping."
            return

        if source.endswith(".zip"):
            tmp_dest_path = dest_path + ".zip"
        else:
            tmp_dest_path = dest_path

        self.download(dest_name, source, tmp_dest_path)

        if tmp_dest_path.endswith(".zip"):
            self.extract(tmp_dest_path, dest_path, filename)

    def file_content_parser(self, path):
        with io.open(path, "r") as f:
            for line in f:
                line = line.strip()
                if len(line) < 1 or line[0] == "#":
                    continue

                yield [e.strip() for e in line.split('\t')]

    def _openfiles(self):
        self.files = {}
        for lang in self.langs:
            dst_file_path = os.path.join(local_settings.TMPDATA_DIR, "django_{0}.po".format(lang))
            self.files[lang] = io.open(dst_file_path, "w")
            self.files[lang].write(PO_HEADER.strip())
            self.files[lang].write(u"\n\n")

    def _closefiles(self):
        for name, f in self.files.iteritems():
            f.close()

    def insert_translation(self, lang, name, translation):
        f = self.files[lang]
        f.write(u'msgid "{0}"\n'.format(name))
        f.write(u'msgstr "{0}"\n\n'.format(translation))

    def parse(self, item):
        if len(item) > 4:
            return

        code, lang = int(item[1]), force_unicode(item[2]).strip()

        if not lang or lang not in self.langs:
            return

        if code not in self.data:
            return

        print item
        self.insert_translation(lang, self.data[code], item[3])

    @transaction.commit_on_success
    def handle(self, *args, **options):
        self.prepare()
        self.download_datafiles()

        dest_name = local_settings.TRANSLATIONS['dest']
        dest_path = os.path.join(local_settings.TMPDATA_DIR, dest_name)

        self._openfiles()

        for item in self.file_content_parser(dest_path):
            self.parse(item)

        self._closefiles()



# -*- coding: utf-8 -*-

from django.template.defaultfilters import slugify
from django.utils.encoding import force_unicode
from django.utils.translation import ugettext_lazy as _
from django.db import models

from south.modelsinspector import add_introspection_rules
from .settings import *


CONTINENT_CHOICES = (
    ('OC', _(u'Oceania')),
    ('EU', _(u'Europe')),
    ('AF', _(u'Africa')),
    ('NA', _(u'North America')),
    ('AN', _(u'Antarctica')),
    ('SA', _(u'South America')),
    ('AS', _(u'Asia')),
)


def slugify_uniquely(value, model, slugfield="slug"):
    """
    Returns a slug on a name which is unique within a model's table
    self.slug = SlugifyUniquely(self.name, self.__class__)
    """
    suffix = 0
    potential = base = slugify(value)
    if len(potential) == 0:
        potential = 'null'
    while True:
        if suffix:
            potential = "-".join([base, str(suffix)])
        if not model.objects.filter(**{slugfield: potential}).count():
            return potential
        suffix += 1


class Base(models.Model):
    """
    Base model with boilerplate for all models.
    """

    geoname_id = models.IntegerField(null=True, blank=True, unique=True)
    slug = models.SlugField(max_length=200, blank=True)
    name = models.CharField(max_length=200, db_index=True)

    class Meta:
        abstract = True

    def __unicode__(self):
        return self.name

    def save(self, *args, **kwargs):
        if not self.slug:
            self.slug = slugify_uniquely(self.name, self.__class__)
        super(Base, self).save(*args, **kwargs)


class Country(Base):
    """
    Country model.
    """

    code2 = models.CharField(max_length=2, null=True, blank=True, unique=True)
    code3 = models.CharField(max_length=3, null=True, blank=True, unique=True)
    continent = models.CharField(max_length=2, db_index=True,
        choices=CONTINENT_CHOICES)
    tld = models.CharField(max_length=5, blank=True, db_index=True)

    class Meta:
        verbose_name_plural = _(u'countries')
        ordering = ['name']

    @property
    def filtered_regions(self):
        if self.code2 in COUNTRIES_WITH_SUBREGION_PROVINCES:
            return self.regions.only_subregions().all()
        return self.regions.only_regions().all()


class RegionManager(models.Manager):
    use_for_related_fields = True

    def only_subregions(self):
        return self.get_query_set()\
            .filter(is_subregion=True)

    def only_regions(self):
        return self.get_query_set()\
            .filter(is_subregion=False)


class Region(Base):
    """
    Region/State model.

    Can search regions or subregions with some helpers
    methods introduced to related manager.

    Examples::

    >>> c = Country.objects.get(code2="ES")
    >>> c.regions.only_subregions()
    [<Region: Ceuta>, <Region: Melilla>, <Region: Murcia>, <Region: Provincia de Castello>, '...']

    >>> c.regions.only_regions()
    [<Region: Andalusia>, <Region: Aragon>, <Region: Asturias>, <Region: Balearic Islands>, '...']
    """

    name_std = models.CharField(max_length=200)
    country = models.ForeignKey(Country, related_name="regions", on_delete = models.CASCADE)
    geoname_code = models.CharField(max_length=50, null=True, blank=True,
        db_index=True)

    is_subregion = models.BooleanField(default=False)
    parent = models.ForeignKey("self", null=True, default=None, related_name="subregions")

    objects = RegionManager()

    class Meta:
        verbose_name = _('region/state')
        verbose_name_plural = _('regions/states')
        ordering = ['name']

    def get_display_name(self):
        return u'%s, %s' % (self.name, self.country.name)

    def to_dict(self):
        return {
            "id": self.pk,
            "name": self.name,
            "slug": self.slug,
            "geoname_id": self.geoname_id,
        }


class City(Base):
    """
    City model.
    """

    name_std = models.CharField(max_length=200, db_index=True)
    alternate_names = models.TextField(null=True, blank=True, default='')

    latitude = models.DecimalField(max_digits=8, decimal_places=5,
        null=True, blank=True)

    longitude = models.DecimalField(max_digits=8, decimal_places=5,
        null=True, blank=True)

    region = models.ForeignKey("Region", blank=True, null=True, related_name="cities")
    subregion = models.ForeignKey("Region", blank=True, null=True, related_name="cities_as_subregion")
    country = models.ForeignKey("Country", related_name="cities", on_delete = models.CASCADE)
    timezone = models.ForeignKey("Timezone", related_name="cities", on_delete = models.CASCADE, null=True, default=None)

    class Meta:
        unique_together = ('country', 'name')
        verbose_name_plural = _(u'cities')
        ordering = ['name']

    def get_display_name(self):
        if self.region_id:
            return u'%s, %s, %s' % (self.name, self.region.name,
                self.country.name)

        return u'%s, %s' % (self.name, self.country.name)

    def to_dict(self):
        return {
            "id": self.pk,
            "name": self.name,
            "slug": self.slug,
        }


class Timezone(models.Model):
    country_code = models.CharField(max_length=100)
    name = models.CharField(max_length=200, unique=True)
    gmt_offset = models.CharField(max_length=50)
    country = models.ForeignKey("Country", related_name="timezones",
        on_delete = models.CASCADE,
        null=True, default=None)

    class Meta:
        ordering = ['name']

    def __unicode__(self):
        return u"{0} (GMT{1})".format(self.name, self.gmt_offset)



# -*- coding: utf-8 -*-

from django.http import HttpResponseBadRequest, HttpResponseNotFound
from .models import Country, Region


class ProvincesForCountry(JsonView):
    def get(self, request):
        if "country" not in request.GET:
            return HttpResponseBadRequest()
        elif not request.GET["country"]:
            return self.render_json({'list': []})

        try:
            country = Country.objects.get(pk=request.GET['country'])
        except Country.DoesNotExist:
            return HttpResponseNotFound()

        return self.render_json({'list': [x.to_dict() for x in country.filtered_regions.all()]})


class CitiesForCountry(JsonView):
    def get(self, request):
        if "country" not in request.GET:
            return HttpResponseBadRequest()
        elif not request.GET["country"]:
            return self.render_json({'list': []})

        try:
            country = Country.objects.get(pk=request.GET['country'])
        except Country.DoesNotExist:
            return HttpResponseNotFound()

        return self.render_json({'list': [x.to_dict() for x in country.cities.all()]})



# -*- coding: utf-8 -*-

from django.contrib import admin
from . import models

admin.site.register(models.Country)
admin.site.register(models.Region)
admin.site.register(models.City)
admin.site.register(models.Timezone)



# -*- coding: utf-8 -*-

import io
import os
import sys
import os.path
import optparse
import zipfile
import requests

from django.core.management.base import BaseCommand
from django.utils.encoding import force_unicode
from django.db import transaction
from django.conf import settings

from ...models import City, Country, Region, Timezone
from ... import settings as local_settings

from os.path import abspath, dirname, join, normpath, exists
from os import mkdir, remove
from subprocess import Popen, PIPE
import tempfile
import shutil
import glob


class Command(BaseCommand):
    temp_path = "/tmp/cities-translations"

    def make_language_directories(self):
        cities_path = normpath(join(dirname(abspath(__file__)), "..", ".."))
        cities_locale_path = join(cities_path, "locale")

        if not exists(cities_locale_path):
            mkdir(cities_locale_path)

        for lang, lang_ in settings.LANGUAGES:
            lang_path = join(cities_locale_path, lang)

            if not exists(lang_path):
                mkdir(lang_path)

            lc_messages_path = join(lang_path, "LC_MESSAGES")
            if not exists(lc_messages_path):
                mkdir(lc_messages_path)

            real_file = join(lc_messages_path, 'django.po')
            yield lang, real_file

    def merge_po_file(self, temp_path, real_path):
        p = Popen(["msgmerge", real_path, temp_path], stdout=PIPE)

        with io.open(real_path + ".tmp", "wt", encoding="utf-8") as f:
            for line in p.stdout:
                f.write(line.decode("utf-8"))

        os.remove(temp_path)
        os.remove(real_path)

        shutil.copy(real_path + ".tmp", real_path)
        os.remove(real_path + ".tmp")

    def generate_new_po(self, lang):
        pathlist = glob.glob(join(self.temp_path, lang) + "*")
        output_path = join(self.temp_path, "{0}_django.po".format(lang))

        command = ["xgettext", "-language=Python", "-keyword=ugettext:1,2",
            "-o", output_path, "--from-code", "utf-8"]
        command += pathlist

        r = Popen(command, stdout=PIPE).wait()
        return output_path

    @transaction.commit_on_success
    def handle(self, *args, **options):
        languages = list(self.make_language_directories())

        # Clear temporal directories
        if exists(self.temp_path):
            shutil.rmtree(self.temp_path)
        mkdir(self.temp_path)

        for lang, real_path in languages:
            # Country
            countrys_path = join(self.temp_path, "{0}.country.py".format(lang))
            with io.open(countrys_path, "wt", encoding="utf-8") as f:
                for country in Country.objects.only('name'):
                    f.write(u'ugettext(u"{0}")\n'.format(country.name))

            regions_path = join(self.temp_path, "{0}.region.py".format(lang))
            with io.open(regions_path, "wt", encoding="utf-8") as f:
                for region in Region.objects.only('name'):
                    f.write(u'ugettext(u"{0}")\n'.format(region.name))

            city_path = join(self.temp_path, "{0}.city.py".format(lang))
            with io.open(city_path, "wt", encoding="utf-8") as f:
                for city in City.objects.only('name'):
                    f.write(u'ugettext(u"{0}")\n'.format(city.name))

            new_po_file = self.generate_new_po(lang)

            if exists(real_path):
                self.merge_po_file(new_po_file, real_path)
            else:
                shutil.copy(new_po_file, real_path)
                os.remove(new_po_file)

        shutil.rmtree(self.temp_path)



from setuptools import setup

description = """
A full database of contry/state/city/timezone for django. Based on django-cities but
with more clean codebase.
"""

setup(
    name = "djcities",
    url = "https://github.com/niwibe/djcities",
    author = "Andrey Antukh",
    author_email = "niwi@niwi.be",
    version='0.1',
    packages = [
        "djcities",
        "djcities.management",
        "djcities.management.commands",
    ],
    description = description.strip(),
    zip_safe=False,
    include_package_data = True,
    classifiers = [
        "Development Status :: 4 - Beta",
        "Operating System :: OS Independent",
        "Environment :: Web Environment",
        "Framework :: Django",
        "License :: OSI Approved :: BSD License",
        "Intended Audience :: Developers",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        #"Programming Language :: Python :: 3",
        #"Programming Language :: Python :: 3.2",
        #"Programming Language :: Python :: 3.3",
        "Topic :: Software Development :: Libraries",
        "Topic :: Utilities",
    ],
)



"""
Settings for this application.

See:

 - http://en.wikipedia.org/wiki/List_of_languages_by_number_of_native_speakers
 - http://download.geonames.org/export/dump/iso-languagecodes.txt


DATA_DIR
    Absolute path to download and extract data into. Default is
    cities_light/data. Overridable in settings.CITIES_LIGHT_DATA_DIR
"""

from django.conf import settings

import os.path
import re

ALPHA_REGEXP = re.compile('[\W_]+', re.UNICODE)

SOURCES = {
    'country': {
        'source': getattr(settings, 'CITIES_COUNTRY_SOURCES',
            'http://download.geonames.org/export/dump/countryInfo.txt'),
        'dest': 'countries.txt',
        'sort': 1,
    },
    'timezone': {
        'source': getattr(settings, 'CITIES_TIMEZONES',
            'http://download.geonames.org/export/dump/timeZones.txt'),
        'dest': 'timezones.txt',
        'sort': 2,
    },
    'region': {
        'source': getattr(settings, 'CITIES_REGION_SOURCES',
            'http://download.geonames.org/export/dump/admin1CodesASCII.txt'),
        'dest': 'regions.txt',
        'sort': 3,
    },
    'subregion': {
        'source': getattr(settings, 'CITIES_SUBREGION_SOURCES',
            'http://download.geonames.org/export/dump/admin2Codes.txt'),
        'dest': 'subregions.txt',
        'sort': 4,
    },

    'city': {
        'source': getattr(settings, 'CITIES_CITY_SOURCES',
            'http://download.geonames.org/export/dump/cities15000.zip'),
        'dest': 'cities.txt',
        'file': 'cities15000.txt',
        'sort': 5,
    },
}

TRANSLATIONS = {
    'source':'http://download.geonames.org/export/dump/alternateNames.zip',
    'file': 'alternateNames.txt',
    'dest': 'translations.txt',
}

TMPDATA_DIR = getattr(settings, "CITIES_TMPDATA_DIR",
    os.path.join("/tmp", "django-cities")
)

# Countries whose provinces are subregions, not regions
COUNTRIES_WITH_SUBREGION_PROVINCES = getattr(settings, "COUNTRIES_WITH_SUBREGION_PROVINCES", ['ES'])



# -*- coding: utf-8 -*-

import io
import os
import sys
import os.path
import optparse
import zipfile
import requests

from django.core.management.base import BaseCommand
from django.utils.encoding import force_unicode
from django.db import transaction, IntegrityError, connection

from ...models import City, Country, Region, Timezone
from ... import settings as local_settings
from django.core.exceptions import MultipleObjectsReturned


class Command(BaseCommand):
    option_list = BaseCommand.option_list + (
        optparse.make_option('--country', action='store', default=None, dest='country',
                             help='Download and import only data for this country.'),)

    def prepare(self):
        if not os.path.exists(local_settings.TMPDATA_DIR):
            print "Creating temporal data directory: {0}".format(local_settings.TMPDATA_DIR)
            os.mkdir(local_settings.TMPDATA_DIR)

    def download(self, name, sourcedata, dest_path):
        response = requests.get(sourcedata['source'])

        with io.open(dest_path, "bw") as f:
            f.write(response.content)
            #for chuck in response.iter_content(512):
            #    f.write(chuck)
            #    sys.stdout.write(".")
            #sys.stdout.write("\n")

    def extract(self, infile, outfile, filename):
        zfile = zipfile.ZipFile(infile)
        if zfile:
             with io.open(outfile, "wb") as f:
                f.write(zfile.read(filename))

    def download_datafiles(self):
        for name, sourcedata in local_settings.SOURCES.iteritems():
            print "Downloading: {0} from {1}".format(name, sourcedata['source'])

            dest_path = os.path.join(local_settings.TMPDATA_DIR, sourcedata['dest'])
            if os.path.exists(dest_path):
                print "File exists, skiping."
                continue

            if sourcedata['source'].endswith(".zip"):
                tmp_dest_path = dest_path + ".zip"
            else:
                tmp_dest_path = dest_path

            self.download(name, sourcedata, tmp_dest_path)

            if tmp_dest_path.endswith(".zip"):
                self.extract(tmp_dest_path, dest_path, sourcedata['file'])

    def file_content_parser(self, path):
        with io.open(path, "r") as f:
            for line in f:
                line = line.strip()
                if len(line) < 1 or line[0] == "#" or line[0] == 'CountryCode':
                    continue

                yield [e.strip() for e in line.split('\t')]

    @transaction.commit_on_success
    def handle(self, *args, **options):

        if "extra_sql" in options:
            extra_sql = options["extra_sql"]
            if extra_sql:
                if not os.path.exists(extra_sql):
                    raise RuntimeError("File {0} does not exists".format(extra_sql))

                cursor = connection.cursor()
                with io.open(extra_sql, "rt") as f:
                    for line in f:
                        if not line or line.strip().startswith('--'):
                            continue
                        print "ex: {0}".format(line.strip())
                        cursor.execute(line)

                cursor.close()

        # Remove old data
        Timezone.objects.all().delete()
        City.objects.all().delete()
        Region.objects.all().delete()
        Country.objects.all().delete()

        self._options = options
        self.prepare()
        self.download_datafiles()

        self.bulk_cities = []
        self.bulk_regions = []
        self.bulk_country = []
        self.bulk_timezone = []

        for name, sourcedata in sorted(local_settings.SOURCES.items(), key=lambda x: x[1]['sort']):
            dest_path = os.path.join(local_settings.TMPDATA_DIR, sourcedata['dest'])
            print "Parsing: {0} - {1}".format(name, dest_path)

            _parse_method = getattr(self, "{0}_import".format(name))

            for item in self.file_content_parser(dest_path):
                _parse_method(item)

    def timezone_import(self, items):
        if len(items) < 3:
            return

        if items[0] == "CountryCode":
            return

        country_code2, name, offset = items[0], items[1], items[2]

        try:
            tz = Timezone.objects.get(name=name)
        except Timezone.DoesNotExist:
            tz = Timezone(name=name)

        tz.country = self._get_country(country_code2)
        tz.country_code = country_code2
        tz.gmt_offset = offset
        tz.save()

    def country_import(self, items):
        geoname_id = items[16]
        code2 = force_unicode(items[0])
        code3 = force_unicode(items[1])
        name = force_unicode(items[4])
        continent = force_unicode(items[8])

        if self._options['country']:
            if code2.lower() != self._options['country'].lower():
                return
        try:
            country = Country.objects.get(code2=code2)
        except Country.DoesNotExist:
            country = Country(code2=code2)

        country.name = name
        country.code3 = code3
        country.continent = continent
        country.tld = items[9:1]

        if geoname_id:
            country.geoname_id = geoname_id

        sem = transaction.savepoint()
        try:
            country.save()
            transaction.savepoint_commit(sem)
        except IntegrityError:
            transaction.savepoint_rollback(sem)

    def region_import(self, items):
        if len(items) < 4:
            return

        items = [force_unicode(x) for x in items]

        raw_code = items[0].split(".")
        name = items[2]
        name_std = items[1]
        geoname_id = items[3]

        country_code2 = raw_code[0]
        geoname_code = raw_code[1]

        if self._options['country']:
            if country_code2.lower() != self._options['country'].lower():
                return

        try:
            country = self._get_country(country_code2)
        except Country.DoesNotExist:
            return

        try:
            region = Region.objects.get(name=name, country=country, is_subregion=False)
        except Region.DoesNotExist:
            region = Region(is_subregion=False, name=name)

        if country is None:
            import pdb; pdb.set_trace()

        region.country = country
        region.geoname_code = geoname_code
        region.name_std = name_std
        region.geoname_id = geoname_id

        sem = transaction.savepoint()
        try:
            region.save()
            transaction.savepoint_commit(sem)
        except IntegrityError:
            transaction.savepoint_rollback(sem)

    def subregion_import(self, items):
        raw_code = force_unicode(items[0]).split(".")
        name = force_unicode(items[2])
        name_std = force_unicode(items[1])
        geoname_id = items[3]

        country_code2 = raw_code[0]
        region_geoname_code = raw_code[1]
        subregion_geoname_code = raw_code[2]

        if self._options['country']:
            if country_code2.lower() != self._options['country'].lower():
                return

        try:
            country = self._get_country(country_code2)
        except Country.DoesNotExist:
            return

        try:
            region = self._get_region(country_code2, region_geoname_code)
        except Region.DoesNotExist:
            region = None

        try:
            subregion = Region.objects.get(geoname_code=subregion_geoname_code,
                            country=country, is_subregion=True)
        except Region.DoesNotExist:
            subregion = Region(is_subregion=True, parent=region)

        subregion.name = name
        subregion.name_std = name_std
        subregion.country = country
        subregion.geoname_code = subregion_geoname_code
        subregion.geoname_id = geoname_id

        sem = transaction.savepoint()
        try:
            subregion.save()
            transaction.savepoint_commit(sem)
        except IntegrityError:
            transaction.savepoint_rollback(sem)

    def city_import(self, items):
        geoname_id = items[0]
        name = force_unicode(items[2])
        name_std = force_unicode(items[1])
        alternate_names = force_unicode(items[3])
        country_code2 = items[8]
        region_geoname_code = items[10]
        subregion_geoname_code = items[11]
        timezone_name = items[17]

        if self._options['country']:
            if items[8].lower() != self._options['country'].lower():
                return

        timezone = Timezone.objects.get(name=timezone_name)
        country = self._get_country(country_code2)
        region = self._get_region(country_code2, region_geoname_code)
        subregion = self._get_sub_region(country_code2, region_geoname_code, subregion_geoname_code)

        try:
            city = City.objects.get(name=name, country=country, region=region, subregion=subregion)
        except City.DoesNotExist:
            city = City(name=name)

        city.region = region
        city.subregion = subregion
        city.latitude = items[4]
        city.longitude = items[5]
        city.timezone = timezone
        city.country = country
        city.name_std = name_std
        city.alternate_names = alternate_names
        city.geoname_id = geoname_id

        sem = transaction.savepoint()
        try:
            city.save()
            transaction.savepoint_commit(sem)
        except IntegrityError:
            transaction.savepoint_rollback(sem)

    def _get_country(self, country_id):
        """
        Simple lazy identity map for country_id->country
        """

        if not hasattr(self, '_country_codes'):
            self._country_codes = {}

        if self._options['country']:
            if country_id.lower() != self._options['country'].lower():
                return None

        if country_id not in self._country_codes.keys():
            self._country_codes[country_id] = Country.objects.get(code2=country_id)

        return self._country_codes[country_id]

    def _get_region(self, country_code2, region_geoname_code):
        if not hasattr(self, '_region_codes'):
            self._region_codes = {}

        if country_code2 not in self._region_codes:
            self._region_codes[country_code2] = {}

        if region_geoname_code not in self._region_codes[country_code2]:
            try:
                self._region_codes[country_code2][region_geoname_code] = Region.objects\
                    .get(country__code2=country_code2, geoname_code=region_geoname_code, is_subregion=False)
            except Region.DoesNotExist:
                return None

        return self._region_codes[country_code2][region_geoname_code]

    def _get_sub_region(self, country_code2, region_geoname_code, subregion_geoname_code):
        if not hasattr(self, '_sub_region_codes'):
            self._sub_region_codes = {}

        if country_code2 not in self._sub_region_codes:
            self._sub_region_codes[country_code2] = {}

        if subregion_geoname_code not in self._sub_region_codes[country_code2]:
            try:
                self._sub_region_codes[country_code2][subregion_geoname_code] = Region.objects\
                    .get(country__code2=country_code2, geoname_code=subregion_geoname_code, is_subregion=True)
            except Region.DoesNotExist:
                return None

        return self._sub_region_codes[country_code2][subregion_geoname_code]
